import { createElement } from 'react';
import { NodeHighlight } from 'node-highlight';

const processData = (data, prefix) => {
    /**
     * Transform kind property of each TreeNode element if exists.
     *
     * By default kind looks like `keyword.language`, it converts to `${prefix}keyword language`.
     *
     * If the current node is a sublanguage it converts `xml` to `language-xml`.
     *
     * @param {TreeNode} node
     * @returns {TreeNode}
     */
    const processChildren = (node) => {
        if (typeof node === 'string') {
            return node;
        }
        // process scope
        const scope = node.kind.split('.').map((_scope, index) => {
            if (!node.subLanguage && index === 0) {
                return `${prefix}${_scope}`;
            }
            if (index === 0) {
                return `language-${_scope}`;
            }
            return _scope;
        }).join(' ');
        node.kind = scope;
        node.children = node.children.map(processChildren);
        return node;
    };
    const _data = data.children.map(processChildren);
    return _data;
};
/**
 * Return an array of both string or React elements. See `Element`.
 *
 * @param node
 * @param prefix `className` prefix.
 * @returns
 */
const buildElements = (node, prefix = 'nhgl-') => {
    let _key = 1;
    const processChildren = (children) => {
        const _root = children;
        if (typeof _root === 'string') {
            return _root;
        }
        const _childs = _root.children.map(processChildren);
        const element = createElement('span', {
            key: `${_key}-${_root.kind}`,
            children: _childs,
            className: _root.kind,
        });
        _key += 1;
        return element;
    };
    return processData(node, prefix).map(processChildren);
};

const _highlight = (code, language, classPrefix = 'nhgl-') => {
    const result = new NodeHighlight({ escapeHTML: false, classPrefix }).highlight(code, language);
    if (!result._status) {
        return [createElement('span', { children: [result.value], key: '', className: `${classPrefix}$${language}` })];
    }
    return buildElements(result._emitter.rootNode, classPrefix);
};
/**
 * Highlight `code` using `node-highlight` and returns an array of React nodes.
 *
 * See `CodeElement` type.
 *
 * @param code code to highlight
 * @param language language name. Must be supported for 'node-highlight'.
 * @param prefix string to pass to `node-highlight`.
 *
 * @returns {Promise<CodeElement[]>} a list of both strings or React elements.
 */
const highlight = async (code, language, prefix) => _highlight(code, language, prefix);
/**
 * Highlight `code` using `node-highlight` and returns an array of React nodes.
 *
 * See `CodeElement` type.
 *
 * @param code code to highlight
 * @param language language name. Must be supported for 'node-highlight'.
 * @param prefix string to pass to `node-highlight`.
 *
 * @returns {CodeElement[]} a list of both strings or React elements.
 */
const highlightSync = (code, language, prefix) => _highlight(code, language, prefix);

const CodeHighlight = ({ language, code, children, className, prefix = 'nhgl-', }) => {
    const _code = code || children.toString();
    const _class = className || `${prefix}${language}`;
    return createElement('pre', { className: _class, children: highlightSync(_code, language, prefix) });
};

export { CodeHighlight, highlight, highlightSync };
